@@ -2,8 +2,8 @@
  *
  * NEBMODS.C - Event Broker Module Functions
  *
- * Copyright (c) 2002-2007 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   12-14-2007
+ * Copyright (c) 2002-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 02-23-2008
  *
  * License:
  *
@@ -35,6 +35,8 @@
 nebmodule *neb_module_list=NULL;
 nebcallback **neb_callback_list=NULL;
 
+extern char     *temp_path;
+
 
 /*#define DEBUG*/
 
@@ -47,7 +49,9 @@ nebcallback **neb_callback_list=NULL;
 
 /* initialize module routines */
 int neb_init_modules(void){
+#ifdef USE_LTDL
 	int result=OK;
+#endif
 
 	/* initialize library */
 #ifdef USE_LTDL
@@ -62,7 +66,9 @@ int neb_init_modules(void){
 
 /* deinitialize module routines */
 int neb_deinit_modules(void){
-	int result;
+#ifdef USE_LTDL
+	int result=OK;
+#endif
 
 	/* deinitialize library */
 #ifdef USE_LTDL
@@ -78,8 +84,8 @@ int neb_deinit_modules(void){
 
 /* add a new module to module list */
 int neb_add_module(char *filename,char *args,int should_be_loaded){
-	nebmodule *new_module;
-	int x;
+	nebmodule *new_module=NULL;
+	int x=OK;
 
 	if(filename==NULL)
 		return ERROR;
@@ -107,9 +113,7 @@ int neb_add_module(char *filename,char *args,int should_be_loaded){
 	new_module->next=neb_module_list;
 	neb_module_list=new_module;
 
-#ifdef DEBUG
-	printf("Added module: name='%s', args='%s', should_be_loaded='%d'\n",filename,args,should_be_loaded);
-#endif
+	log_debug_info(DEBUGL_EVENTBROKER,0,"Added module: name='%s', args='%s', should_be_loaded='%d'\n",filename,args,should_be_loaded);
 
 	return OK;
         }
@@ -117,17 +121,17 @@ int neb_add_module(char *filename,char *args,int should_be_loaded){
 
 /* free memory allocated to module list */
 int neb_free_module_list(void){
-	nebmodule *temp_module;
-	nebmodule *next_module;
-	int x;
+	nebmodule *temp_module=NULL;
+	nebmodule *next_module=NULL;
+	int x=OK;
 
 	for(temp_module=neb_module_list;temp_module;){
 		next_module=temp_module->next;
-		free(temp_module->filename);
-		free(temp_module->args);
+		my_free(temp_module->filename);
+		my_free(temp_module->args);
 		for(x=0;x<NEBMODULE_MODINFO_NUMITEMS;x++)
-			free(temp_module->info[x]);
-		free(temp_module);
+			my_free(temp_module->info[x]);
+		my_free(temp_module);
 		temp_module=next_module;
 	        }
 
@@ -147,8 +151,8 @@ int neb_free_module_list(void){
 
 /* load all modules */
 int neb_load_all_modules(void){
-	nebmodule *temp_module;
-	int result;
+	nebmodule *temp_module=NULL;
+	int result=OK;
 
 	for(temp_module=neb_module_list;temp_module;temp_module=temp_module->next){
 		result=neb_load_module(temp_module);
@@ -160,14 +164,18 @@ int neb_load_all_modules(void){
 
 /* load a particular module */
 int neb_load_module(nebmodule *mod){
-	char temp_buffer[MAX_INPUT_BUFFER];
 	int (*initfunc)(int,char *,void *);
-	int *module_version_ptr;
-	int result;
+	int *module_version_ptr=NULL;
+	char *output_file=NULL;
+	int dest_fd=-1;
+	int source_fd=-1;
+	char buffer[MAX_INPUT_BUFFER]={0};
+	int bytes_read=0;
+	int result=OK;
 
 	if(mod==NULL || mod->filename==NULL)
 		return ERROR;
-
+	
 	/* don't reopen the module */
 	if(mod->is_currently_loaded==TRUE)
 		return OK;
@@ -176,21 +184,51 @@ int neb_load_module(nebmodule *mod){
 	if(mod->should_be_loaded==FALSE)
 		return ERROR;
 
-	/* load the module */
+	/********** 
+	   Using dlopen() is great, but a real danger as-is.  The problem with loaded modules is that if you overwrite the original file (e.g. using 'mv'),
+	   you do not alter the inode of the original file.  Since the original file/module is memory-mapped in some fashion, Nagios will segfault the next
+	   time an event broker call is directed to one of the module's callback functions.  This is extremely problematic when it comes to upgrading NEB
+	   modules while Nagios is running.  A workaround is to (1) 'mv' the original/loaded module file to another name (on the same filesystem)
+	   and (2) copy the new module file to the location of the original one (using the original filename).  In this scenario, dlopen() will keep referencing
+	   the original file/inode for callbacks.  This is not an ideal solution.   A better one is to delete the module file once it is loaded by dlopen().
+	   This prevents other processed from unintentially overwriting the original file, which would cause Nagios to crash.  However, if we delete the file
+	   before anyone else can muck with it, things should be good.  'lsof' shows that a deleted file is still referenced by the kernel and callback
+	   functions continue to work once the module has been loaded.  Long story, but this took quite a while to figure out, as there isn't much 
+	   of anything I could find on the subject other than some sketchy info on similar problems on HP-UX.  Hopefully this will save future coders some time.
+	   So... the trick is to (1) copy the module to a temp file, (2) dlopen() the temp file, and (3) immediately delete the temp file. 
+	************/
+
+	/* open a temp file for copying the module */
+	asprintf(&output_file,"%s/nebmodXXXXXX",temp_path);
+	if((dest_fd=mkstemp(output_file))==-1){
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Could not safely copy module '%s'.  The module will not be loaded: %s\n",mod->filename,strerror(errno));
+		return ERROR;
+		}
+	/* open module file for reading and copy it */
+	if((source_fd=open(mod->filename,O_RDONLY,0644))>0){
+		while((bytes_read=read(source_fd,buffer,sizeof(buffer)))>0)
+			write(dest_fd,buffer,bytes_read);
+		close(source_fd);
+		close(dest_fd);
+		}
+	else{
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Could not safely copy module '%s'.  The module will not be loaded: %s\n",mod->filename,strerror(errno));
+		return ERROR;
+		}
+
+	/* load the module (use the temp copy we just made) */
 #ifdef USE_LTDL
-	mod->module_handle=lt_dlopen(mod->filename);
+	mod->module_handle=lt_dlopen(output_file);
 #else
-	mod->module_handle=dlopen(mod->filename,RTLD_NOW|RTLD_GLOBAL);
+	mod->module_handle=(void *)dlopen(output_file,RTLD_NOW|RTLD_GLOBAL);
 #endif
 	if(mod->module_handle==NULL){
 
 #ifdef USE_LTDL
-		snprintf(temp_buffer,sizeof(temp_buffer),"Error: Could not load module '%s' -> %s\n",mod->filename,lt_dlerror());
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Could not load module '%s' -> %s\n",mod->filename,lt_dlerror());
 #else
-		snprintf(temp_buffer,sizeof(temp_buffer),"Error: Could not load module '%s' -> %s\n",mod->filename,dlerror());
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Could not load module '%s' -> %s\n",mod->filename,dlerror());
 #endif
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_RUNTIME_ERROR);
 
 		return ERROR;
 	        }
@@ -198,6 +236,17 @@ int neb_load_module(nebmodule *mod){
 	/* mark the module as being loaded */
 	mod->is_currently_loaded=TRUE;
 
+	/* delete the temp copy of the module we just created and loaded */
+	/* this will prevent other processes from overwriting the file (using the same inode), which would cause Nagios to crash */
+	/* the kernel will keep the deleted file in memory until we unload it */
+	/* NOTE: This *should* be portable to most Unices, but I've only tested it on Linux */
+	if(unlink(output_file)==-1){
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Could not delete temporary file '%s' used for module '%s'.  The module will be unloaded: %s\n",output_file,mod->filename,strerror(errno));
+		neb_unload_module(mod,NEBMODULE_FORCE_UNLOAD,NEBMODULE_ERROR_API_VERSION);
+
+		return ERROR;
+		}
+
 	/* find module API version */
 #ifdef USE_LTDL
 	module_version_ptr=(int *)lt_dlsym(mod->module_handle,"__neb_api_version");
@@ -208,9 +257,7 @@ int neb_load_module(nebmodule *mod){
 	/* check the module API version */
 	if(module_version_ptr==NULL || ((*module_version_ptr)!=CURRENT_NEB_API_VERSION)){
 
-		snprintf(temp_buffer,sizeof(temp_buffer),"Error: Module '%s' is using an old or unspecified version of the event broker API.  Module will be unloaded.\n",mod->filename);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_RUNTIME_ERROR);
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Module '%s' is using an old or unspecified version of the event broker API.  Module will be unloaded.\n",mod->filename);
 
 		neb_unload_module(mod,NEBMODULE_FORCE_UNLOAD,NEBMODULE_ERROR_API_VERSION);
 
@@ -221,15 +268,13 @@ int neb_load_module(nebmodule *mod){
 #ifdef USE_LTDL
 	mod->init_func=lt_dlsym(mod->module_handle,"nebmodule_init");
 #else
-	mod->init_func=dlsym(mod->module_handle,"nebmodule_init");
+	mod->init_func=(void *)dlsym(mod->module_handle,"nebmodule_init");
 #endif
 
 	/* if the init function could not be located, unload the module */
 	if(mod->init_func==NULL){
 
-		snprintf(temp_buffer,sizeof(temp_buffer),"Error: Could not locate nebmodule_init() in module '%s'.  Module will be unloaded.\n",mod->filename);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_RUNTIME_ERROR);
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Could not locate nebmodule_init() in module '%s'.  Module will be unloaded.\n",mod->filename);
 
 		neb_unload_module(mod,NEBMODULE_FORCE_UNLOAD,NEBMODULE_ERROR_NO_INIT);
 
@@ -243,31 +288,25 @@ int neb_load_module(nebmodule *mod){
 	/* if the init function returned an error, unload the module */
 	if(result!=OK){
 
-		snprintf(temp_buffer,sizeof(temp_buffer),"Error: Function nebmodule_init() in module '%s' returned an error.  Module will be unloaded.\n",mod->filename);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_RUNTIME_ERROR);
+		logit(NSLOG_RUNTIME_ERROR,FALSE,"Error: Function nebmodule_init() in module '%s' returned an error.  Module will be unloaded.\n",mod->filename);
 
 		neb_unload_module(mod,NEBMODULE_FORCE_UNLOAD,NEBMODULE_ERROR_BAD_INIT);
 
 		return ERROR;
 	        }
 
-	snprintf(temp_buffer,sizeof(temp_buffer),"Event broker module '%s' initialized successfully.\n",mod->filename);
-	temp_buffer[sizeof(temp_buffer)-1]='\x0';
-	write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
+	logit(NSLOG_INFO_MESSAGE,FALSE,"Event broker module '%s' initialized successfully.\n",mod->filename);
 
 	/* locate the de-initialization function (may or may not be present) */
 #ifdef USE_LTDL
 	mod->deinit_func=lt_dlsym(mod->module_handle,"nebmodule_deinit");
 #else
-	mod->deinit_func=dlsym(mod->module_handle,"nebmodule_deinit");
+	mod->deinit_func=(void *)dlsym(mod->module_handle,"nebmodule_deinit");
 #endif
 
-#ifdef DEBUG
-	printf("Module '%s' loaded with return code of '%d'\n",mod->filename,result);
+	log_debug_info(DEBUGL_EVENTBROKER,0,"Module '%s' loaded with return code of '%d'\n",mod->filename,result);
 	if(mod->deinit_func!=NULL)
-		printf("\tnebmodule_deinit() found\n");
-#endif
+		log_debug_info(DEBUGL_EVENTBROKER,0,"nebmodule_deinit() found\n");
 
 	return OK;
         }
@@ -298,16 +337,13 @@ int neb_unload_all_modules(int flags, int reason){
 
 /* close (unload) a particular module */
 int neb_unload_module(nebmodule *mod, int flags, int reason){
-	char temp_buffer[MAX_INPUT_BUFFER];
 	int (*deinitfunc)(int,int);
-	int result;
+	int result=OK;
 
 	if(mod==NULL)
 		return ERROR;
 
-#ifdef DEBUG
-	printf("Attempting to unload module '%s': flags=%d, reason=%d\n",mod->filename,flags,reason);
-#endif
+	log_debug_info(DEBUGL_EVENTBROKER,0,"Attempting to unload module '%s': flags=%d, reason=%d\n",mod->filename,flags,reason);
 
 	/* call the de-initialization function if available (and the module was initialized) */
 	if(mod->deinit_func && reason!=NEBMODULE_ERROR_BAD_INIT){
@@ -335,13 +371,9 @@ int neb_unload_module(nebmodule *mod, int flags, int reason){
 	/* mark the module as being unloaded */
 	mod->is_currently_loaded=FALSE;
 
-#ifdef DEBUG
-	printf("Module '%s' unloaded successfully.\n",mod->filename);
-#endif
+	log_debug_info(DEBUGL_EVENTBROKER,0,"Module '%s' unloaded successfully.\n",mod->filename);
 
-	snprintf(temp_buffer,sizeof(temp_buffer),"Event broker module '%s' deinitialized successfully.\n",mod->filename);
-	temp_buffer[sizeof(temp_buffer)-1]='\x0';
-	write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
+	logit(NSLOG_INFO_MESSAGE,FALSE,"Event broker module '%s' deinitialized successfully.\n",mod->filename);
 
 	return OK;
         }
@@ -375,7 +407,7 @@ int neb_set_module_info(void *handle, int type, char *data){
 		return NEBERROR_BADMODULEHANDLE;
 
 	/* free any previously allocated memory */
-	free(temp_module->info[type]);
+	my_free(temp_module->info[type]);
 
 	/* allocate memory for the new data */
 	if((temp_module->info[type]=(char *)strdup(data))==NULL)
@@ -386,7 +418,6 @@ int neb_set_module_info(void *handle, int type, char *data){
 
 
 
-
 /****************************************************************************/
 /****************************************************************************/
 /* CALLBACK FUNCTIONS                                                       */
@@ -395,10 +426,10 @@ int neb_set_module_info(void *handle, int type, char *data){
 
 /* allows a module to register a callback function */
 int neb_register_callback(int callback_type, void *mod_handle, int priority, int (*callback_func)(int,void *)){
-	nebmodule *temp_module;
-	nebcallback *new_callback;
-	nebcallback *temp_callback;
-	nebcallback *last_callback;
+	nebmodule *temp_module=NULL;
+	nebcallback *new_callback=NULL;
+	nebcallback *temp_callback=NULL;
+	nebcallback *last_callback=NULL;
 
 	if(callback_func==NULL)
 		return NEBERROR_NOCALLBACKFUNC;
@@ -460,9 +491,9 @@ int neb_register_callback(int callback_type, void *mod_handle, int priority, int
 
 /* dregisters all callback functions for a given module */
 int neb_deregister_module_callbacks(nebmodule *mod){
-	nebcallback *temp_callback;
-	nebcallback *next_callback;
-	int callback_type;
+	nebcallback *temp_callback=NULL;
+	nebcallback *next_callback=NULL;
+	int callback_type=0;
 
 	if(mod==NULL)
 		return NEBERROR_NOMODULE;
@@ -485,8 +516,8 @@ int neb_deregister_module_callbacks(nebmodule *mod){
 
 /* allows a module to deregister a callback function */
 int neb_deregister_callback(int callback_type, int (*callback_func)(int,void *)){
-	nebcallback *temp_callback;
-	nebcallback *last_callback;
+	nebcallback *temp_callback=NULL;
+	nebcallback *last_callback=NULL;
 	nebcallback *next_callback=NULL;
 
 	if(callback_func==NULL)
@@ -520,7 +551,7 @@ int neb_deregister_callback(int callback_type, int (*callback_func)(int,void *))
 			neb_callback_list[callback_type]=NULL;
 		else
 			last_callback->next=next_callback;
-		free(temp_callback);
+		my_free(temp_callback);
 		}
 	
 	return OK;
@@ -530,9 +561,10 @@ int neb_deregister_callback(int callback_type, int (*callback_func)(int,void *))
 
 /* make callbacks to modules */
 int neb_make_callbacks(int callback_type, void *data){
-	nebcallback *temp_callback;
+	nebcallback *temp_callback=NULL;
 	int (*callbackfunc)(int,void *);
-	int cbresult;
+	register int cbresult=0;
+	int total_callbacks=0;
 
 	/* make sure callback list is initialized */
 	if(neb_callback_list==NULL)
@@ -542,23 +574,34 @@ int neb_make_callbacks(int callback_type, void *data){
 	if(callback_type<0 || callback_type>=NEBCALLBACK_NUMITEMS)
 		return ERROR;
 
+	log_debug_info(DEBUGL_EVENTBROKER,1,"Making callbacks (type %d)...\n",callback_type);
+
 	/* make the callbacks... */
 	for(temp_callback=neb_callback_list[callback_type];temp_callback!=NULL;temp_callback=temp_callback->next){
 		callbackfunc=temp_callback->callback_func;
 		cbresult=callbackfunc(callback_type,data);
-#ifdef DEBUG
-		printf("Callback type %d resulted in return code of %d\n",callback_type,cbresult);
-#endif
+
+		total_callbacks++;
+		log_debug_info(DEBUGL_EVENTBROKER,2,"Callback #%d (type %d) return code = %d\n",total_callbacks,callback_type,cbresult);
+
+		/* module wants to cancel callbacks to other modules (and potentially cancel the default Nagios handling of an event) */
+		if(cbresult==NEBERROR_CALLBACKCANCEL)
+			break;
+
+		/* module wants to override default Nagios handling of an event */
+		/* not sure if we should bail out here just because one module wants to override things - what about other modules? EG 12/11/2006 */
+		else if(cbresult==NEBERROR_CALLBACKOVERRIDE)
+			break;
 	        }
 
-	return OK;
+	return cbresult;
         }
 
 
 
 /* initialize callback list */
 int neb_init_callback_list(void){
-	int x;
+	register int x=0;
 
 	/* allocate memory for the callback list */
 	neb_callback_list=(nebcallback **)malloc(NEBCALLBACK_NUMITEMS*sizeof(nebcallback *));
@@ -575,9 +618,9 @@ int neb_init_callback_list(void){
 
 /* free memory allocated to callback list */
 int neb_free_callback_list(void){
-	nebcallback *temp_callback;
-	nebcallback *next_callback;
-	int x;
+	nebcallback *temp_callback=NULL;
+	nebcallback *next_callback=NULL;
+	register int x=0;
 
 	if(neb_callback_list==NULL)
 		return OK;
@@ -586,14 +629,13 @@ int neb_free_callback_list(void){
 
 		for(temp_callback=neb_callback_list[x];temp_callback!=NULL;temp_callback=next_callback){
 			next_callback=temp_callback->next;
-			free(temp_callback);
+			my_free(temp_callback);
 	                }
 
 		neb_callback_list[x]=NULL;
 	        }
 
-	free(neb_callback_list);
-	neb_callback_list=NULL;
+	my_free(neb_callback_list);
 
 	return OK;
         }
