@@ -2,8 +2,8 @@
  *
  * STATUSDATA.C - External status data for Nagios CGIs
  *
- * Copyright (c) 2000-2005 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   07-25-2005
+ * Copyright (c) 2000-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified:   10-19-2006
  *
  * License:
  *
@@ -56,6 +56,8 @@ servicestatus   *servicestatus_list_tail=NULL;
 
 hoststatus      **hoststatus_hashlist=NULL;
 servicestatus   **servicestatus_hashlist=NULL;
+
+extern int      use_pending_states;
 #endif
 
 
@@ -81,7 +83,7 @@ int initialize_status_data(char *config_file){
 
 /* update all status data (aggregated dump) */
 int update_all_status_data(void){
-	int result;
+	int result=OK;
 
 #ifdef USE_EVENT_BROKER
 	/* send data to event broker */
@@ -177,6 +179,25 @@ int update_service_status(service *svc,int aggregated_dump){
 	return OK;
         }
 
+
+
+/* updates contact status info */
+int update_contact_status(contact *cntct,int aggregated_dump){
+
+#ifdef USE_EVENT_BROKER
+	/* send data to event broker (non-aggregated dumps only) */
+	if(aggregated_dump==FALSE)
+	        broker_contact_status(NEBTYPE_CONTACTSTATUS_UPDATE,NEBFLAG_NONE,NEBATTR_NONE,cntct,NULL);
+#endif
+
+	/* currently a noop if aggregated updates is TRUE */
+
+	/* update all status data if we're not aggregating updates */
+	if(aggregate_status_updates==FALSE)
+		update_all_status_data();
+
+	return OK;
+        }
 #endif
 
 
@@ -213,12 +234,13 @@ int read_status_data(char *config_file,int options){
 
 /* adds hoststatus to hash list in memory */
 int add_hoststatus_to_hashlist(hoststatus *new_hoststatus){
-	hoststatus *temp_hoststatus, *lastpointer;
-	int hashslot;
+	hoststatus *temp_hoststatus=NULL;
+	hoststatus *lastpointer=NULL;
+	int hashslot=0;
+	int i=0;
 
 	/* initialize hash list */
 	if(hoststatus_hashlist==NULL){
-		int i;
 
 		hoststatus_hashlist=(hoststatus **)malloc(sizeof(hoststatus *)*HOSTSTATUS_HASHSLOTS);
 		if(hoststatus_hashlist==NULL)
@@ -231,12 +253,12 @@ int add_hoststatus_to_hashlist(hoststatus *new_hoststatus){
 	if(!new_hoststatus)
 		return 0;
 
-	hashslot=hashfunc1(new_hoststatus->host_name,HOSTSTATUS_HASHSLOTS);
+	hashslot=hashfunc(new_hoststatus->host_name,NULL,HOSTSTATUS_HASHSLOTS);
 	lastpointer=NULL;
-	for(temp_hoststatus=hoststatus_hashlist[hashslot];temp_hoststatus && compare_hashdata1(temp_hoststatus->host_name,new_hoststatus->host_name)<0;temp_hoststatus=temp_hoststatus->nexthash)
+	for(temp_hoststatus=hoststatus_hashlist[hashslot];temp_hoststatus && compare_hashdata(temp_hoststatus->host_name,NULL,new_hoststatus->host_name,NULL)<0;temp_hoststatus=temp_hoststatus->nexthash)
 		lastpointer=temp_hoststatus;
 
-	if(!temp_hoststatus || (compare_hashdata1(temp_hoststatus->host_name,new_hoststatus->host_name)!=0)){
+	if(!temp_hoststatus || (compare_hashdata(temp_hoststatus->host_name,NULL,new_hoststatus->host_name,NULL)!=0)){
 		if(lastpointer)
 			lastpointer->nexthash=new_hoststatus;
 		else
@@ -252,12 +274,12 @@ int add_hoststatus_to_hashlist(hoststatus *new_hoststatus){
 
 
 int add_servicestatus_to_hashlist(servicestatus *new_servicestatus){
-	servicestatus *temp_servicestatus, *lastpointer;
-	int hashslot;
+	servicestatus *temp_servicestatus=NULL, *lastpointer=NULL;
+	int hashslot=0;
+	int i=0;
 
 	/* initialize hash list */
 	if(servicestatus_hashlist==NULL){
-		int i;
 
 		servicestatus_hashlist=(servicestatus **)malloc(sizeof(servicestatus *)*SERVICESTATUS_HASHSLOTS);
 		if(servicestatus_hashlist==NULL)
@@ -270,12 +292,12 @@ int add_servicestatus_to_hashlist(servicestatus *new_servicestatus){
 	if(!new_servicestatus)
 		return 0;
 
-	hashslot=hashfunc2(new_servicestatus->host_name,new_servicestatus->description,SERVICESTATUS_HASHSLOTS);
+	hashslot=hashfunc(new_servicestatus->host_name,new_servicestatus->description,SERVICESTATUS_HASHSLOTS);
 	lastpointer=NULL;
-	for(temp_servicestatus=servicestatus_hashlist[hashslot];temp_servicestatus && compare_hashdata2(temp_servicestatus->host_name,temp_servicestatus->description,new_servicestatus->host_name,new_servicestatus->description)<0;temp_servicestatus=temp_servicestatus->nexthash)
+	for(temp_servicestatus=servicestatus_hashlist[hashslot];temp_servicestatus && compare_hashdata(temp_servicestatus->host_name,temp_servicestatus->description,new_servicestatus->host_name,new_servicestatus->description)<0;temp_servicestatus=temp_servicestatus->nexthash)
 		lastpointer=temp_servicestatus;
 
-	if(!temp_servicestatus || (compare_hashdata2(temp_servicestatus->host_name,temp_servicestatus->description,new_servicestatus->host_name,new_servicestatus->description)!=0)){
+	if(!temp_servicestatus || (compare_hashdata(temp_servicestatus->host_name,temp_servicestatus->description,new_servicestatus->host_name,new_servicestatus->description)!=0)){
 		if(lastpointer)
 			lastpointer->nexthash=new_servicestatus;
 		else
@@ -299,7 +321,6 @@ int add_servicestatus_to_hashlist(servicestatus *new_servicestatus){
 
 /* adds a host status entry to the list in memory */
 int add_host_status(hoststatus *new_hoststatus){
-	char temp_buffer[MAX_INPUT_BUFFER];
 	char date_string[MAX_DATETIME_LENGTH];
 
 	/* make sure we have what we need */
@@ -325,16 +346,23 @@ int add_host_status(hoststatus *new_hoststatus){
 			break;
 		        }
 		if(new_hoststatus->has_been_checked==FALSE){
-			new_hoststatus->status=HOST_PENDING;
-			free(new_hoststatus->plugin_output);
+			if(use_pending_states==TRUE)
+				new_hoststatus->status=HOST_PENDING;
+			my_free(new_hoststatus->plugin_output);
 			if(new_hoststatus->should_be_scheduled==TRUE){
 				get_time_string(&new_hoststatus->next_check,date_string,sizeof(date_string),LONG_DATE_TIME);
-				snprintf(temp_buffer,sizeof(temp_buffer)-1,"Host check scheduled for %s",date_string);
-				temp_buffer[sizeof(temp_buffer)-1]='\x0';
-				new_hoststatus->plugin_output=strdup(temp_buffer);
+				asprintf(&new_hoststatus->plugin_output,"Host check scheduled for %s",date_string);
 			        }
-			else
-				new_hoststatus->plugin_output=strdup("Host has not been checked yet");
+			else{
+				/* passive-only hosts that have just been scheduled for a forced check */
+				if(new_hoststatus->checks_enabled==FALSE && new_hoststatus->next_check!=(time_t)0L && (new_hoststatus->check_options & CHECK_OPTION_FORCE_EXECUTION)){
+					get_time_string(&new_hoststatus->next_check,date_string,sizeof(date_string),LONG_DATE_TIME);
+					asprintf(&new_hoststatus->plugin_output,"Forced host check scheduled for %s",date_string);
+					}
+				/* passive-only hosts not scheduled to be checked */
+				else
+					new_hoststatus->plugin_output=(char *)strdup("Host is not scheduled to be checked...");
+				}
 		        }
 	        }
 
@@ -361,7 +389,6 @@ int add_host_status(hoststatus *new_hoststatus){
 
 /* adds a service status entry to the list in memory */
 int add_service_status(servicestatus *new_svcstatus){
-	char temp_buffer[MAX_INPUT_BUFFER];
 	char date_string[MAX_DATETIME_LENGTH];
 
 	/* make sure we have what we need */
@@ -391,16 +418,23 @@ int add_service_status(servicestatus *new_svcstatus){
 			break;
 		        }
 		if(new_svcstatus->has_been_checked==FALSE){
-			new_svcstatus->status=SERVICE_PENDING;
-			free(new_svcstatus->plugin_output);
+			if(use_pending_states==TRUE)
+				new_svcstatus->status=SERVICE_PENDING;
+			my_free(new_svcstatus->plugin_output);
 			if(new_svcstatus->should_be_scheduled==TRUE){
 				get_time_string(&new_svcstatus->next_check,date_string,sizeof(date_string),LONG_DATE_TIME);
-				snprintf(temp_buffer,sizeof(temp_buffer)-1,"Service check scheduled for %s",date_string);
-				temp_buffer[sizeof(temp_buffer)-1]='\x0';
-				new_svcstatus->plugin_output=strdup(temp_buffer);
+				asprintf(&new_svcstatus->plugin_output,"Service check scheduled for %s",date_string);
 			        }
-			else
-				new_svcstatus->plugin_output=strdup("Service is not scheduled to be checked...");
+			else{
+				/* passive-only services that have just been scheduled for a forced check */
+				if(new_svcstatus->checks_enabled==FALSE && new_svcstatus->next_check!=(time_t)0L && (new_svcstatus->check_options & CHECK_OPTION_FORCE_EXECUTION)){
+					get_time_string(&new_svcstatus->next_check,date_string,sizeof(date_string),LONG_DATE_TIME);
+					asprintf(&new_svcstatus->plugin_output,"Forced service check scheduled for %s",date_string);
+					}
+				/* passive-only services not scheduled to be checked */
+				else
+					new_svcstatus->plugin_output=(char *)strdup("Service is not scheduled to be checked...");
+				}
 		        }
 	        }
 
@@ -435,35 +469,35 @@ int add_service_status(servicestatus *new_svcstatus){
 
 /* free all memory for status data */
 void free_status_data(void){
-	hoststatus *this_hoststatus;
-	hoststatus *next_hoststatus;
-	servicestatus *this_svcstatus;
-	servicestatus *next_svcstatus;
+	hoststatus *this_hoststatus=NULL;
+	hoststatus *next_hoststatus=NULL;
+	servicestatus *this_svcstatus=NULL;
+	servicestatus *next_svcstatus=NULL;
 
 	/* free memory for the host status list */
 	for(this_hoststatus=hoststatus_list;this_hoststatus!=NULL;this_hoststatus=next_hoststatus){
 		next_hoststatus=this_hoststatus->next;
-		free(this_hoststatus->host_name);
-		free(this_hoststatus->plugin_output);
-		free(this_hoststatus->perf_data);
-		free(this_hoststatus);
+		my_free(this_hoststatus->host_name);
+		my_free(this_hoststatus->plugin_output);
+		my_free(this_hoststatus->long_plugin_output);
+		my_free(this_hoststatus->perf_data);
+		my_free(this_hoststatus);
 	        }
 
 	/* free memory for the service status list */
 	for(this_svcstatus=servicestatus_list;this_svcstatus!=NULL;this_svcstatus=next_svcstatus){
 		next_svcstatus=this_svcstatus->next;
-		free(this_svcstatus->host_name);
-		free(this_svcstatus->description);
-		free(this_svcstatus->plugin_output);
-		free(this_svcstatus->perf_data);
-		free(this_svcstatus);
+		my_free(this_svcstatus->host_name);
+		my_free(this_svcstatus->description);
+		my_free(this_svcstatus->plugin_output);
+		my_free(this_svcstatus->long_plugin_output);
+		my_free(this_svcstatus->perf_data);
+		my_free(this_svcstatus);
 	        }
 
 	/* free hash lists reset list pointers */
-	free(hoststatus_hashlist);
-	free(servicestatus_hashlist);
-	hoststatus_hashlist=NULL;
-	servicestatus_hashlist=NULL;
+	my_free(hoststatus_hashlist);
+	my_free(servicestatus_hashlist);
 	hoststatus_list=NULL;
 	servicestatus_list=NULL;
 
@@ -480,14 +514,14 @@ void free_status_data(void){
 
 /* find a host status entry */
 hoststatus *find_hoststatus(char *host_name){
-	hoststatus *temp_hoststatus;
+	hoststatus *temp_hoststatus=NULL;
 
 	if(host_name==NULL || hoststatus_hashlist==NULL)
 		return NULL;
 
-	for(temp_hoststatus=hoststatus_hashlist[hashfunc1(host_name,HOSTSTATUS_HASHSLOTS)];temp_hoststatus && compare_hashdata1(temp_hoststatus->host_name,host_name)<0;temp_hoststatus=temp_hoststatus->nexthash);
+	for(temp_hoststatus=hoststatus_hashlist[hashfunc(host_name,NULL,HOSTSTATUS_HASHSLOTS)];temp_hoststatus && compare_hashdata(temp_hoststatus->host_name,NULL,host_name,NULL)<0;temp_hoststatus=temp_hoststatus->nexthash);
 
-	if(temp_hoststatus && (compare_hashdata1(temp_hoststatus->host_name,host_name)==0))
+	if(temp_hoststatus && (compare_hashdata(temp_hoststatus->host_name,NULL,host_name,NULL)==0))
 		return temp_hoststatus;
 
 	return NULL;
@@ -496,14 +530,14 @@ hoststatus *find_hoststatus(char *host_name){
 
 /* find a service status entry */
 servicestatus *find_servicestatus(char *host_name,char *svc_desc){
-	servicestatus *temp_servicestatus;
+	servicestatus *temp_servicestatus=NULL;
 
 	if(host_name==NULL || svc_desc==NULL || servicestatus_hashlist==NULL)
 		return NULL;
 
-	for(temp_servicestatus=servicestatus_hashlist[hashfunc2(host_name,svc_desc,SERVICESTATUS_HASHSLOTS)];temp_servicestatus && compare_hashdata2(temp_servicestatus->host_name,temp_servicestatus->description,host_name,svc_desc)<0;temp_servicestatus=temp_servicestatus->nexthash);
+	for(temp_servicestatus=servicestatus_hashlist[hashfunc(host_name,svc_desc,SERVICESTATUS_HASHSLOTS)];temp_servicestatus && compare_hashdata(temp_servicestatus->host_name,temp_servicestatus->description,host_name,svc_desc)<0;temp_servicestatus=temp_servicestatus->nexthash);
 
-	if(temp_servicestatus && (compare_hashdata2(temp_servicestatus->host_name,temp_servicestatus->description,host_name,svc_desc)==0))
+	if(temp_servicestatus && (compare_hashdata(temp_servicestatus->host_name,temp_servicestatus->description,host_name,svc_desc)==0))
 		return temp_servicestatus;
 
 	return NULL;
@@ -519,7 +553,7 @@ servicestatus *find_servicestatus(char *host_name,char *svc_desc){
 
 /* gets the total number of services of a certain state for a specific host */
 int get_servicestatus_count(char *host_name, int type){
-	servicestatus *temp_status;
+	servicestatus *temp_status=NULL;
 	int count=0;
 
 	if(host_name==NULL)
